"""Cell class"""

"""
Copyright (c) 2022, EPFL/Blue Brain Project

 This file is part of BluePyEfe <https://github.com/BlueBrain/BluePyEfe>

 This library is free software; you can redistribute it and/or modify it under
 the terms of the GNU Lesser General Public License version 3.0 as published
 by the Free Software Foundation.

 This library is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 details.

 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""
import logging
import numpy
import matplotlib.pyplot as plt
import pathlib

from bluepyefe.ecode import eCodes
from bluepyefe.reader import *
from bluepyefe.plotting import _save_fig

logger = logging.getLogger(__name__)


class Cell(object):

    """Contains the metadata related to a cell as well as the
    electrophysiological recordings once they are read"""

    def __init__(self, name):
        """
        Constructor

        Args:
            name (str): name of the cell.
        """

        self.name = name

        self.recordings = []
        self.rheobase = None

    def reader(self, config_data, recording_reader=None):
        """Define the reader method used to read the ephys data for the
        present recording and returns the data contained in the file.

        Args:
            config_data (dict): metadata for the recording considered.
            recording_reader (callable or None): method that will be used to
                read the files containing the recordings. If None, the function
                used will be chosen automatically based on the extension
                of the file.
        """

        if "v_file" in config_data:
            filename = config_data["v_file"]
        elif "filepath" in config_data:
            filename = config_data["filepath"]

        if recording_reader:
            return recording_reader(config_data)
        if ".abf" in filename:
            return axon_reader(config_data)
        if ".ibw" in filename or ".bwav" in filename:
            return igor_reader(config_data)
        if ".nwb" in filename:
            return nwb_reader(config_data)

        raise Exception(
            "The format of the ephys files is unknown and no custom reader"
            "were provided."
        )

    def get_protocol_names(self):
        """List of all the protocols available for the present cell."""

        return list(set([rec.protocol_name for rec in self.recordings]))

    def get_recordings_by_protocol_name(self, protocol_name):
        """List of all the recordings available for the present cell for a
        given protocol.

        Args:
            protocol_name (str): name of the protocol for which to get
                the recordings.
        """

        return [
            rec
            for rec in self.recordings
            if rec.protocol_name == protocol_name
        ]

    def get_recordings_id_by_protocol_name(self, protocol_name):
        """List of the indexes of the recordings available for the present
        cell for a given protocol.

        Args:
            protocol_name (str): name of the protocol for which to get
                the recordings.
        """

        return [
            i
            for i, trace in enumerate(self.recordings)
            if trace.protocol_name == protocol_name
        ]

    def read_recordings(
        self,
        protocol_data,
        protocol_name,
        recording_reader=None,
        efel_settings=None
    ):
        """
        For each member of a list of recordings metadata, instantiate a Recording object and
        populate it by reading the matching data file.

        Args:
            protocol_data (list of dict): list of metadata for the
                recordings considered.
            protocol_name (str): name of the protocol for which to get
                the recordings.
            recording_reader (callable or None): method that will be used to
                read the files containing the recordings. If None, the function
                used will be chosen automatically based on the extension
                of the file.
            efel_settings (dict): eFEL settings in the form
                {setting_name: setting_value}.
        """

        for config_data in protocol_data:

            if "protocol_name" not in config_data:
                config_data["protocol_name"] = protocol_name

            for reader_data in self.reader(config_data, recording_reader):

                for ecode in eCodes.keys():
                    if ecode.lower() in protocol_name.lower():
                        rec = eCodes[ecode](
                            config_data,
                            reader_data,
                            protocol_name,
                            efel_settings
                        )
                        self.recordings.append(rec)
                        break
                else:
                    raise KeyError(
                        f"There is no eCode linked to the stimulus name "
                        f"{protocol_name.lower()}. See ecode/__init__.py for "
                        f"the available stimuli names"
                    )

    def extract_efeatures(
        self,
        protocol_name,
        efeatures,
        efeature_names=None,
        efel_settings=None
    ):
        """
        Extract the efeatures for the recordings matching the protocol name.

        Args:
            protocol_name (str): name of the protocol for which to extract
                the efeatures.
            efeatures (list of str): name of the efeatures to extract from
                the recordings.
            efeature_names (list of str): Optional. Given name for the
                features. Can and should be used if the same feature
                is to be extracted several time on different sections
                of the same recording.
        """

        for i in self.get_recordings_id_by_protocol_name(protocol_name):
            self.recordings[i].compute_efeatures(
                efeatures, efeature_names, efel_settings)

    def compute_relative_amp(self):
        """Compute the relative current amplitude for all the recordings as a
        percentage of the rheobase."""

        if self.rheobase not in (0.0, None, False, numpy.nan):

            for i in range(len(self.recordings)):
                self.recordings[i].compute_relative_amp(self.rheobase)

        else:

            logger.warning(
                "Cannot compute the relative current amplitude for the "
                "recordings of cell {} because its rheobase is {}."
                "".format(
                    self.name, self.rheobase
                )
            )
            self.rheobase = None

    def plot_recordings(self, protocol_name, output_dir=None, show=False):
        """Plot all the recordings matching a protocol name

        Args:
            protocol_name (str): name of the protocol for which to plot the
                recordings.
            output_dir (str): path to the output directory to which to save
                the figures.
            show (bool): should the figures be displayed in addition to
                being saved.
        """

        recordings = self.get_recordings_by_protocol_name(protocol_name)

        if not len(recordings):
            return None, None

        recordings_amp = [rec.amp for rec in recordings]
        recordings = [recordings[k] for k in numpy.argsort(recordings_amp)]

        n_cols = 6
        n_rows = int(2 * numpy.ceil(len(recordings) / n_cols))

        fig, axs = plt.subplots(
            n_rows, n_cols,
            figsize=[3.0 + 3.0 * int(n_cols), 2.5 * n_rows],
            squeeze=False
        )

        for i, rec in enumerate(recordings):

            col = i % int(n_cols)
            row = 2 * int(i / n_cols)

            display_ylabel = col == 0
            display_xlabel = row + 1 == axs.shape[0]

            _, _ = rec.plot(
                axis_current=axs[row][col],
                axis_voltage=axs[row + 1][col],
                display_xlabel=display_xlabel,
                display_ylabel=display_ylabel
            )

        fig.suptitle("Cell: {}, Experiment: {}".format(self.name, protocol_name))

        plt.subplots_adjust(wspace=0.53, hspace=0.7)

        for ax in axs.flatten():
            if not ax.lines:
                ax.set_visible(False)

        # Do not use tight-layout, it significantly increases the runtime
        plt.margins(0, 0)

        if show:
            fig.show()

        if output_dir is not None:
            filename = "{}_{}_recordings.pdf".format(self.name, protocol_name)
            dirname = pathlib.Path(output_dir) / self.name
            _save_fig(dirname, filename)

        return fig, axs

    def plot_all_recordings(self, output_dir=None, show=False):
        """Plot all the recordings of the cell.

        Args:
            output_dir (str): path to the output directory to which to save
                the figures.
            show (bool): should the figures be displayed in addition to
                being saved.
        """

        for protocol_name in self.get_protocol_names():
            self.plot_recordings(protocol_name, output_dir, show=show)



"""EFeatureTarget class"""

"""
Copyright (c) 2022, EPFL/Blue Brain Project

 This file is part of BluePyEfe <https://github.com/BlueBrain/BluePyEfe>

 This library is free software; you can redistribute it and/or modify it under
 the terms of the GNU Lesser General Public License version 3.0 as published
 by the Free Software Foundation.

 This library is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 details.

 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""
import numpy
import logging

logger = logging.getLogger(__name__)


class EFeatureTarget():

    """E-feature target defined by an efeature to compute for a given protocol
    and amplitude. Contains the resulting values"""

    def __init__(
            self,
            efeature_name,
            efel_feature_name,
            protocol_name,
            amplitude,
            tolerance,
            efel_settings=None,

    ):
        """Constructor.

        Args:
            efeature_name (str): name of the feature (can be different than
                the efel_feature_name - e.g. Spikecount_phase1)
            efel_feature_name (str): name of the eFeature in the eFEL library
                (ex: 'AP1_peak')
            protocol_name (str): name of the recording from which the efeature
                should be computed
            amplitude (float): amplitude of the current stimuli for which the
                efeature should be computed (expressed as a percentage of the
                threshold amplitude (rheobase))
            tolerance (float): tolerance around the target amplitude in which
                an experimental recording will be seen as a hit during
                efeatures extraction (expressed as a percentage of the
                threshold amplitude (rheobase))
            efel_settings (dict): target specific efel settings.
        """

        self.efeature_name = efeature_name
        if self.efeature_name is None:
            self.efeature_name = efel_feature_name
        self.efel_feature_name = efel_feature_name
        self.protocol_name = protocol_name

        self.amplitude = amplitude
        self.tolerance = tolerance

        self.efel_settings = efel_settings
        if self.efel_settings is None:
            self.efel_settings = {}

        self._values = []
        self._files = []
        self._auto_thresholds = []

    @property
    def mean(self):
        """Average of the e-feature value at target"""

        return numpy.nanmean(self._values)

    @property
    def std(self):
        """Standard deviation of the e-feature value at target"""

        return numpy.nanstd(self._values)

    @property
    def sample_size(self):
        """Number of value that matched the target present"""

        return len(self._values)

    def append(self, value, files=None):
        """Append a feature value to the present target"""

        if not isinstance(value, (int, float)):
            raise TypeError("Expected value of type int or float")

        if numpy.isnan(value) or value is None:
            logger.warning(
                "Trying to append {} to efeature {} for protocol {} {}. Value "
                "will be ignored".format(
                    value,
                    self.efel_feature_name,
                    self.protocol_name,
                    self.amplitude
                )
            )
            return

        self._values.append(value)
        if files:
            self._files += files

    def clear(self):
        """Clear the list of feature values"""

        self._values = []
        self._files = []

    def add_effective_threshold(self):
        """If auto threshold detection was used during feature extraction,
        update the efel settings with the Threshold that was actually used"""

        if self._auto_thresholds:
            self.efel_settings["Threshold"] = numpy.median(self._auto_thresholds)

    def as_dict(self, save_files_used=False, default_std_value=1e-3):
        """Returns the target in the form of a dictionary in a legacy format"""

        self.add_effective_threshold()

        std = self.std
        if std == 0.0:
            logger.warning(
                "Standard deviation for efeatures {} stimulus {} is 0 and"
                "will be set to {}".format(
                    self.efel_feature_name, self.protocol_name, default_std_value
                )
            )
            std = default_std_value

        feature_dict = {
            "feature": self.efel_feature_name,
            "val": [self.mean, std],
            "n": self.sample_size,
            "efel_settings": self.efel_settings
        }

        if self.efeature_name:
            feature_dict['efeature_name'] = self.efeature_name
        if save_files_used:
            feature_dict['files'] = self.files

        return feature_dict

    def __str__(self):
        """String representation"""

        str_form = "Target E-Feature {} for protocol {} {:.1f}%:\n".format(
            self.efel_feature_name, self.protocol_name, self.amplitude
        )

        str_form += "Sample size (n): {}".format(self.sample_size)

        if self.sample_size:
            str_form += "\nMean: {:.5f}\nStandard deviation: {:.5f}".format(
                self.mean, self.std
            )

        return str_form



"""Step eCode class"""

"""
Copyright (c) 2022, EPFL/Blue Brain Project

 This file is part of BluePyEfe <https://github.com/BlueBrain/BluePyEfe>

 This library is free software; you can redistribute it and/or modify it under
 the terms of the GNU Lesser General Public License version 3.0 as published
 by the Free Software Foundation.

 This library is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 details.

 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""
import logging
import numpy

from ..recording import Recording
from .tools import base_current
from .tools import scipy_signal2d

logger = logging.getLogger(__name__)


class Step(Recording):
    def __init__(
        self,
        config_data,
        reader_data,
        protocol_name="step",
        efel_settings=None
    ):

        super(Step, self).__init__(config_data, reader_data, protocol_name)

        self.ton = None
        self.toff = None
        self.tend = None
        self.amp = None
        self.hypamp = None
        self.dt = None

        self.amp_rel = None
        self.hypamp_rel = None

        if self.t is not None and self.current is not None:
            self.interpret(
                self.t, self.current, self.config_data, self.reader_data
            )

        if self.voltage is not None:
            self.compute_spikecount(efel_settings)

        self.export_attr = ["ton", "toff", "tend", "amp", "hypamp", "dt",
                            "amp_rel", "hypamp_rel"]

    def get_stimulus_parameters(self):
        """Returns the eCode parameters"""
        ecode_params = {
            "delay": self.ton,
            "amp": self.amp,
            "thresh_perc": self.amp_rel,
            "duration": self.toff - self.ton,
            "totduration": self.tend,
        }
        return ecode_params

    def interpret(self, t, current, config_data, reader_data):
        """Analyse a current with a step and extract from it the parameters
        needed to reconstruct the array"""
        self.dt = t[1]

        # Smooth the current
        smooth_current = None

        # Set the threshold to detect the step
        noise_level = numpy.std(numpy.concatenate((self.current[:50], self.current[-50:])))
        step_threshold = numpy.clip(2. * noise_level, 1e-5, numpy.max(self.current))

        # The buffer prevent miss-detection of the step when artifacts are
        # present at the very start or very end of the current trace
        buffer_detect = 2.0
        idx_buffer = int(buffer_detect / self.dt)
        idx_buffer = max(1, idx_buffer)

        if "ton" in config_data and config_data["ton"] is not None:
            self.ton = int(round(config_data["ton"] / self.dt))
        elif "ton" in reader_data and reader_data["ton"] is not None:
            self.ton = int(round(reader_data["ton"]))
        else:
            self.ton = None

        # toff (index, not ms)
        if "toff" in config_data and config_data["toff"] is not None:
            self.toff = int(round(config_data["toff"] / self.dt))
        elif "toff" in reader_data and reader_data["toff"] is not None:
            self.toff = int(round(reader_data["toff"]))
        else:
            self.toff = None

        # amp
        if "amp" in config_data and config_data["amp"] is not None:
            self.amp = config_data["amp"]
        elif "amp" in reader_data and reader_data["amp"] is not None:
            self.amp = reader_data["amp"]
        else:
            self.amp = None

        # hypamp
        if "hypamp" in config_data and config_data["hypamp"] is not None:
            self.hypamp = config_data["hypamp"]
        elif "hypamp" in reader_data and reader_data["hypamp"] is not None:
            self.hypamp = reader_data["hypamp"]

        # Infer the begin and end of the step current
        if self.ton is None:
            if self.hypamp is None:
                self.hypamp = base_current(current)
            if smooth_current is None:
                smooth_current = scipy_signal2d(current, 85)
            _ = numpy.abs(smooth_current[idx_buffer:] - self.hypamp)
            self.ton = idx_buffer + numpy.argmax(_ > step_threshold)

        else:
            # Infer the base current hypamp
            self.hypamp = base_current(current, idx_ton=self.ton)

        if self.toff is None:
            if smooth_current is None:
                smooth_current = scipy_signal2d(current, 85)
            _ = numpy.flip(
                numpy.abs(smooth_current[:-idx_buffer] - self.hypamp)
            )
            self.toff = (
                (len(current) - numpy.argmax(_ > step_threshold)) - 1 - idx_buffer
            )

        # Get the amplitude of the step current (relative to hypamp)
        if self.amp is None:
            self.amp = (
                numpy.median(current[self.ton : self.toff]) - self.hypamp
            )

        # Converting back ton and toff to ms
        self.ton = t[int(round(self.ton))]
        self.toff = t[int(round(self.toff))]

        self.tend = len(t) * self.dt

        # Check for some common step detection failures when the current
        # is constant.
        if self.ton > self.toff or self.ton > self.tend or \
                self.toff > self.tend:

            self.ton = 0.
            self.toff = self.tend

            logger.warning(
                "The automatic step detection failed for the recording "
                f"{self.protocol_name} in files {self.files}. You should "
                "specify ton and toff by hand in your files_metadata "
                "for this file."
            )

    def generate(self):
        """Generate the step current array from the parameters of the ecode"""
        ton_idx = int(self.ton / self.dt)
        toff_idx = int(self.toff / self.dt)

        t = numpy.arange(0.0, self.tend, self.dt)
        current = numpy.full(t.shape, self.hypamp)
        current[ton_idx:toff_idx] += self.amp

        return t, current



"""Cheops eCode"""

"""
Copyright (c) 2022, EPFL/Blue Brain Project

 This file is part of BluePyEfe <https://github.com/BlueBrain/BluePyEfe>

 This library is free software; you can redistribute it and/or modify it under
 the terms of the GNU Lesser General Public License version 3.0 as published
 by the Free Software Foundation.

 This library is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 details.

 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""
import logging
import numpy

from ..recording import Recording
from .tools import scipy_signal2d

logger = logging.getLogger(__name__)


class NegCheops(Recording):
    def __init__(
        self,
        config_data,
        reader_data,
        protocol_name="NegCheops",
        efel_settings=None
    ):

        super(NegCheops, self).__init__(
            config_data, reader_data, protocol_name
        )

        self.ton = None
        self.t1 = None
        self.t2 = None
        self.t3 = None
        self.t4 = None
        self.tend = None
        self.amp = None
        self.hypamp = None
        self.dt = None

        self.amp_rel = None
        self.hypamp_rel = None

        if self.t is not None and self.current is not None:
            self.interpret(
                self.t, self.current, self.config_data, self.reader_data
            )

        if self.voltage is not None:
            self.compute_spikecount(efel_settings)

        self.export_attr = ["ton", "t1", "t2", "t3", "t4", "toff", "tend",
                            "amp", "hypamp", "dt", "amp_rel", "hypamp_rel"]

    def get_stimulus_parameters(self):
        """Returns the eCode parameters"""
        ecode_params = {
            "delay": self.ton,
            "amp": self.amp,
            "thresh_perc": self.amp_rel,
            "duration": self.toff - self.ton,
            "totduration": self.tend,
        }
        return ecode_params

    def interpret(self, t, current, config_data, reader_data):
        """Analyse a current array and extract from it the parameters
        needed to reconstruct the array"""
        self.dt = t[1]

        # Smooth the current
        smooth_current = scipy_signal2d(current, 85)

        self.set_timing_ecode(
            ["ton", "t1", "t2", "t3", "t4", "toff"], config_data)

        hypamp_value = numpy.median(smooth_current[: self.ton])
        self.set_amplitudes_ecode("hypamp", config_data, reader_data, hypamp_value)

        amp_value = numpy.min(smooth_current[:]) - self.hypamp
        self.set_amplitudes_ecode("amp", config_data, reader_data, amp_value)

        # Converting back to ms
        for name_timing in ["ton", "t1", "t2", "t3", "t4", "toff"]:
            self.index_to_ms(name_timing, t)
        self.tend = len(t) * self.dt

    def generate(self):
        """Generate the current array from the parameters of the ecode"""

        ton = int(self.ton / self.dt)
        t1 = int(self.t1 / self.dt)
        t2 = int(self.t2 / self.dt)
        t3 = int(self.t3 / self.dt)
        t4 = int(self.t4 / self.dt)
        toff = int(self.toff / self.dt)

        time = numpy.arange(0.0, self.tend, self.dt)
        current = numpy.full(time.shape, self.hypamp)

        # First peak
        mid = int(0.5 * (ton + t1))
        current[ton:mid] += numpy.linspace(0.0, self.amp, mid - ton)
        current[mid:t1] += numpy.linspace(self.amp, 0.0, t1 - mid)

        # Second peak
        mid = int(0.5 * (t2 + t3))
        current[t2:mid] += numpy.linspace(0.0, self.amp, mid - t2)
        current[mid:t3] += numpy.linspace(self.amp, 0.0, t3 - mid)

        # Third peak
        mid = int(0.5 * (t4 + toff))
        current[t4:mid] += numpy.linspace(0.0, self.amp, mid - t4)
        current[mid:toff] += numpy.linspace(self.amp, 0.0, toff - mid)

        return time, current



"""Step eCode class"""

"""
Copyright (c) 2022, EPFL/Blue Brain Project

 This file is part of BluePyEfe <https://github.com/BlueBrain/BluePyEfe>

 This library is free software; you can redistribute it and/or modify it under
 the terms of the GNU Lesser General Public License version 3.0 as published
 by the Free Software Foundation.

 This library is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 details.

 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""
import logging
import numpy

from ..recording import Recording
from .tools import base_current
from .tools import scipy_signal2d

logger = logging.getLogger(__name__)


def group_indexes(values, gap=10):
    """Return a list of clusters from a list where consecutive
    values follow each other forming clusters
    eg: [12, 14, 15, 20, 56, 60, 61, 62, 63] -> [[12, 14, 15, 20], [56, 60,
    61, 62, 63]]"""

    clusters = []

    for v in values:
        if not (len(clusters)) or clusters[-1][-1] + gap < v:
            clusters.append([v])
        else:
            clusters[-1].append(v)

    return clusters


def detect_spike(amp, hypamp, smooth_current, dt):

    tspike = []
    duration = []
    delta = []

    threshold = hypamp + (0.1 * amp)
    candidate_spikes = numpy.argwhere(smooth_current > threshold).flatten()
    candidate_spikes = group_indexes(candidate_spikes, gap=10)

    for spike in candidate_spikes:
        tspike.append(spike[0] - 1)
        duration.append(spike[-1] - spike[0] + 1)

    if len(tspike) > 1:
        for i in range(1, len(tspike)):
            end = tspike[i - 1] + duration[i - 1]
            start = tspike[i]
            delta.append(dt * (start - end))

    tspike = numpy.asarray(tspike) * dt
    duration = numpy.mean(numpy.asarray(duration) * dt)
    delta = numpy.mean(delta)

    return tspike, duration, delta


class SpikeRec(Recording):
    def __init__(
        self,
        config_data,
        reader_data,
        protocol_name="SpikeRec",
        efel_settings=None
    ):

        super(SpikeRec, self).__init__(config_data, reader_data, protocol_name)

        self.tend = None
        self.tspike = []
        self.spike_duration = None  # in ms
        self.amp = None
        self.hypamp = None
        self.dt = None
        self.delta = None  # Time difference between two spikes

        self.amp_rel = None
        self.hypamp_rel = None

        if self.t is not None and self.current is not None:
            self.interpret(
                self.t, self.current, self.config_data, self.reader_data
            )

        if self.voltage is not None:
            self.compute_spikecount(efel_settings)

        self.export_attr = ["tend", "tspike", "spike_duration", "delta",
                            "amp", "hypamp", "dt", "amp_rel", "hypamp_rel"]

    @property
    def ton(self):
        return 0.0

    @property
    def toff(self):
        return self.tend

    def get_stimulus_parameters(self):
        """Returns the eCode parameters"""

        ecode_params = {
            "delay": self.tspike[0],
            "n_spikes": len(self.tspike),
            "delta": self.delta,
            "amp": self.amp,
            "thresh_perc": self.amp_rel,
            "spike_duration": self.spike_duration,
            "totduration": self.tend,
        }

        return ecode_params

    def interpret(self, t, current, config_data, reader_data):
        """Analyse a current with a step and extract from it the parameters
        needed to reconstruct the array"""
        self.dt = t[1]

        # Smooth the current
        smooth_current = scipy_signal2d(current, 15)

        hypamp_value = base_current(current)
        self.set_amplitudes_ecode("hypamp", config_data, reader_data, hypamp_value)

        amp_value = numpy.max(smooth_current)
        self.set_amplitudes_ecode("amp", config_data, reader_data, amp_value)

        # Get the beginning and end of the spikes
        if (
            not len(self.tspike)
            or self.spike_duration is None
            or self.delta is None
        ):
            self.tspike, self.spike_duration, self.delta = detect_spike(
                self.amp, self.hypamp, smooth_current, self.dt
            )

        self.tend = len(t) * self.dt

    def generate(self):
        """Generate the step current array from the parameters of the ecode"""

        t = numpy.arange(0.0, self.tend, self.dt)
        current = numpy.full(t.shape, self.hypamp)

        spike_start = int(self.tspike[0] / self.dt)
        spike_end = int((self.tspike[0] + self.spike_duration) / self.dt)
        current[spike_start:spike_end] += self.amp

        for i in range(1, len(self.tspike)):
            spike_start = int(spike_end + (self.delta / self.dt))
            spike_end = spike_start + int(self.spike_duration / self.dt)
            current[spike_start:spike_end] += self.amp

        return t, current

    def in_target(self, target, tolerance, absolute_amplitude):
        """Returns a boolean. True if the delta of the eCode is close to
        target and False otherwise."""
        logger.warning(
            "The eCode SpikeRec uses delta between current spikes "
            "in ms as target, not amplitude"
        )
        if numpy.abs(target - self.delta) < tolerance:
            return True
        else:
            return False
